%% tclp__errors.pl
%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                                  TCLP                                     %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/yquem/contraintes/coquery/cvs/contraintes/tclpchr/src/tclp/tclp__errors.pl,v 1.7 2005/05/25 13:26:22 coquery Exp $

:- module(tclp__errors, [ print_error/2, 
                          print_error/1, 
                          warning/1,
                          print_exception/1
                        ]).

:- use_module(tclp(tclp__reader)).
:- use_module(library(lists)).
:- use_module(tclp(tclp__utils)).
:- use_module(tclp(tclp__printer)).

%% printer for errors
error_printer(printer('! ')).

%% printer for warnings
warning_printer(printer('* ')).

%% warning(Message) emits a warning message
warning(Message) :-
        warning_printer(WPrinter),
        pprint(WPrinter,['Warning : '|Message]).

%% print_error(Location, Message)
%% prints an error message for an error that occured at location
%% if Message is a list, write its elements one after another, otherwise,
%% write it directly.
print_error(Loc, Message) :-
        line_number(Loc, Line),
        get_filename(Loc, FileName),
        error_printer(ErrPrinter),
        pprint(ErrPrinter,
               ['Error in "',FileName,'", line ',Line,' : '|Message]),
        throw(tclp__error).

%% print_error(Message)
%% prints an error message
%% if Message is a list, write its elements one after another, otherwise,
%% write it directly.
print_error(Message) :-
        error_printer(ErrPrinter),
        pprint(ErrPrinter,
               ['Error : '|Message]),
        throw(tclp__error).        

%%print_syntax_tokens([]).
%%print_syntax_tokens([ atom(X)-_ | Rest ]) :- !,
%%        write_term(X,[quoted(true)]),write(' '),
%%        print_syntax_tokens(Rest).
%%print_syntax_tokens([ var(_,Codes,_)-_ | Rest]) :- !,
%%        atom_codes(Name,Codes),
%%        write(Name), write(' '),
%%        print_syntax_tokens(Rest).
%%print_syntax_tokens([X-_ | Rest ]) :-
%%        write(X),write(' '),
%%        print_syntax_tokens(Rest).

syntax_tokens_print_list([],[]).
syntax_tokens_print_list([ atom(X)-_ | Rest ],
                         ['$quoted'(X),' '| RestList]) :- !,
        syntax_tokens_print_list(Rest,RestList).
syntax_tokens_print_list([ var(_,Codes,_)-_ | Rest],
                         ['$write_option'(Name,[]),' '|RestList]) :- !,
        atom_codes(Name,Codes),
        syntax_tokens_print_list(Rest,RestList).
syntax_tokens_print_list([X-_ | Rest ],
                         ['$write_option'(X,[]),' '|RestList]) :-
        syntax_tokens_print_list(Rest,RestList).

%% pretty prints errors
print_exception(tclp__error_in_file(FileName,
                                    syntax_error(_,Line,Description,
                                                 Terms,NbRemaining))) :-
        length(Trailing, NbRemaining),!,
        append(Start,Trailing,Terms),
        Begining = [ 'Error in "',FileName,'", line ',Line,
                     ': syntax error','\n',
                     Description,'\n'
                    ],
        syntax_tokens_print_list(Start,StartPrintList),
        syntax_tokens_print_list(Trailing,TrailingPrintList),
        multi_append([Begining,StartPrintList,['\n','*HERE*','\n'],
                      TrailingPrintList],PrintList),
        error_printer(ErrPrinter),
        pprint(ErrPrinter,PrintList).
%%        write('! Syntax error'),nl,
%%        write('! '),write(Description),nl,
%%        write('! '),write('in file '),write_term(FileName,[quoted(true)]),
%%        write(', line '), write(Line),nl,
%%        write('! '),print_syntax_tokens(Start),nl,
%%        write('! *HERE*'),nl,
%%        write('! '),print_syntax_tokens(Trailing),nl.
print_exception(filenotfound(FileName)) :-
        error_printer(ErrPrinter),
        pprint(ErrPrinter,[ 'Can''t open "', FileName, '"' ]).
print_exception(E) :-
        error_printer(ErrPrinter),
        pprint(ErrPrinter,['$quoted'(E)]).

